@@ -22,6 +22,7 @@
 #include <fcntl.h>
 #include <inttypes.h>
 #include <dirent.h>
+#include <limits.h>
 
 #include <linux/err.h> /* ERR_PTR */
 #include <linux/if_link.h>
@@ -311,6 +312,25 @@ static size_t bpf_program__insn_cnt(const struct bpf_program *prog)
 }
 #endif
 
+#ifndef HAVE_LIBBPF_BPF_PROGRAM__TYPE
+static inline enum bpf_prog_type bpf_program__type(const struct bpf_program *prog)
+{
+	return bpf_program__get_type((struct bpf_program *)prog);
+}
+#endif
+
+#ifndef HAVE_LIBBPF_BPF_PROGRAM__FLAGS
+static __u32 bpf_program__flags(__unused const struct bpf_program *prog)
+{
+	/* When libbpf doesn't support this we can't get the real value.
+	 * Returning 0 works because the callers check for the presence of a
+	 * specific flag (BPF_F_XDP_HAS_FRAGS), and having it always-off
+	 * disables the frags functionality which is what we want.
+	 */
+	return 0;
+}
+#endif
+
 /* This function has been deprecated in libbpf, but we expose an API that uses
  * section names, so we reimplement it to keep compatibility
  */
@@ -707,6 +727,12 @@ bool xdp_program__xdp_frags_support(const struct xdp_program *prog)
 	return !!(bpf_program__flags(prog->bpf_prog) & BPF_F_XDP_HAS_FRAGS);
 }
 
+#ifndef HAVE_LIBBPF_BPF_PROGRAM__FLAGS
+int xdp_program__set_xdp_frags_support(__unused struct xdp_program *prog, __unused bool frags)
+{
+	return libxdp_err(-EOPNOTSUPP);
+}
+#else
 int xdp_program__set_xdp_frags_support(struct xdp_program *prog, bool frags)
 {
 	__u32 prog_flags;
@@ -728,6 +754,7 @@ int xdp_program__set_xdp_frags_support(struct xdp_program *prog, bool frags)
 
 	return ret;
 }
+#endif // HAVE_LIBBPF_BPF_PROGRAM__FLAGS
 
 const char *xdp_program__name(const struct xdp_program *prog)
 {
@@ -1406,9 +1433,10 @@ struct xdp_program *xdp_program__from_id(__u32 id)
 	}
 
 	prog = xdp_program__from_fd(fd);
+	// duplicated fd already in prog, close original
+	close(fd);
 	if (IS_ERR(prog)) {
 		err = errno;
-		close(fd);
 		errno = err;
 	}
 	return prog;
@@ -1428,9 +1456,10 @@ struct xdp_program *xdp_program__from_pin(const char *pin_path)
 	}
 
 	prog = xdp_program__from_fd(fd);
+	// duplicated fd already in prog, close original
+	close(fd);
 	if (IS_ERR(prog)) {
 		err = errno;
-		close(fd);
 		errno = err;
 	}
 	return prog;
@@ -1583,9 +1612,11 @@ static int xdp_program__load(struct xdp_program *prog)
 		 */
 		prog->is_frags = xdp_program__xdp_frags_support(prog);
 
+#ifdef HAVE_LIBBPF_BPF_PROGRAM__FLAGS
 		if (bpf_program__type(prog->bpf_prog) == BPF_PROG_TYPE_EXT)
 			bpf_program__set_flags(prog->bpf_prog,
 					       bpf_program__flags(prog->bpf_prog) & ~BPF_F_XDP_HAS_FRAGS);
+#endif
 
 		err = bpf_object__load(prog->bpf_obj);
 		if (err)
@@ -1616,6 +1647,39 @@ struct xdp_program *xdp_program__clone(struct xdp_program *prog, unsigned int fl
 					    prog->prog_name, true);
 }
 
+#ifndef HAVE_LIBBPF_BPF_PROGRAM__FLAGS
+static bool kernel_has_frags_support(void)
+{
+	pr_debug("Can't support frags with old version of libbpf that doesn't support setting program flags.\n");
+	return false;
+}
+#else
+static bool kernel_has_frags_support(void)
+{
+	struct xdp_program *test_prog;
+	bool ret = false;
+	int err;
+
+	pr_debug("Checking for kernel frags support\n");
+	test_prog = __xdp_program__find_file("xdp-dispatcher.o", NULL, "xdp_pass", NULL);
+	if (IS_ERR(test_prog)) {
+		err = PTR_ERR(test_prog);
+		pr_warn("Couldn't open BPF file xdp-dispatcher.o\n");
+		return false;
+	}
+
+	bpf_program__set_flags(test_prog->bpf_prog, BPF_F_XDP_HAS_FRAGS);
+	err = xdp_program__load(test_prog);
+	if (!err) {
+		pr_debug("Kernel supports XDP programs with frags\n");
+		ret = true;
+	} else {
+		pr_debug("Kernel DOES NOT support XDP programs with frags\n");
+	}
+	xdp_program__close(test_prog);
+	return ret;
+}
+#endif // HAVE_LIBBPF_BPF_PROGRAM__FLAGS
 
 static int xdp_program__attach_single(struct xdp_program *prog, int ifindex,
 				      enum xdp_attach_mode mode)
@@ -1623,6 +1687,9 @@ static int xdp_program__attach_single(struct xdp_program *prog, int ifindex,
 	int err;
 
 	if (prog->prog_fd < 0) {
+		if (!kernel_has_frags_support())
+			xdp_program__set_xdp_frags_support(prog, false);
+
 		bpf_program__set_type(prog->bpf_prog, BPF_PROG_TYPE_XDP);
 		err = xdp_program__load(prog);
 		if (err)
@@ -2018,32 +2085,6 @@ int xdp_program__test_run(struct xdp_program *prog, struct bpf_test_run_opts *op
 	return libxdp_err(err);
 }
 
-static int xdp_multiprog__check_kernel_frags_support(struct xdp_multiprog *mp)
-{
-	struct xdp_program *test_prog;
-	int err;
-
-	pr_debug("Checking for kernel frags support\n");
-	test_prog = __xdp_program__find_file("xdp-dispatcher.o", NULL, "xdp_pass", NULL);
-	if (IS_ERR(test_prog)) {
-		err = PTR_ERR(test_prog);
-		pr_warn("Couldn't open BPF file xdp-dispatcher.o\n");
-		return err;
-	}
-
-	bpf_program__set_flags(test_prog->bpf_prog, BPF_F_XDP_HAS_FRAGS);
-	err = xdp_program__load(test_prog);
-	if (!err) {
-		pr_debug("Kernel supports XDP programs with frags\n");
-		mp->kernel_frags_support = true;
-	} else {
-		pr_debug("Kernel DOES NOT support XDP programs with frags\n");
-	}
-	xdp_program__close(test_prog);
-
-	return 0;
-}
-
 void xdp_multiprog__close(struct xdp_multiprog *mp)
 {
 	struct xdp_program *p, *next = NULL;
@@ -2452,6 +2493,11 @@ static struct xdp_multiprog *xdp_multiprog__from_id(__u32 id, __u32 hw_id,
 		err = PTR_ERR(mp);
 		goto err;
 	}
+	// duplicated fd/hw_fd already in prog, close originals
+	if (fd > 0)
+		close(fd);
+	if (hw_fd > 0)
+		close(hw_fd);
 	return mp;
 err:
 	if (fd > 0)
@@ -2892,9 +2938,7 @@ static struct xdp_multiprog *xdp_multiprog__generate(struct xdp_program **progs,
 	if (IS_ERR(mp))
 		return mp;
 
-	err = xdp_multiprog__check_kernel_frags_support(mp);
-	if (err)
-		goto err;
+	mp->kernel_frags_support = kernel_has_frags_support();
 
 	if (old_mp) {
 		struct xdp_program *prog;
@@ -3179,6 +3223,51 @@ out:
 	return err;
 }
 
+static int xdp_detach_link(__u32 ifindex, __u32 prog_id) {
+	struct bpf_link_info link_info;
+	__u32 link_info_len, id = 0;
+	int err, fd;
+
+	while (true) {
+		err = bpf_link_get_next_id(id, &id);
+		if (err) {
+			err = -errno;
+			pr_debug("Can't get next link for id %u: %s", id, strerror(errno));
+			return err;
+		}
+
+		fd = bpf_link_get_fd_by_id(id);
+		if (fd < 0) {
+			err = -errno;
+			pr_debug("Can't get link by id %u: %s", id, strerror(errno));
+			return err;
+		}
+
+		memset(&link_info, 0, sizeof(link_info));
+		link_info_len = sizeof(link_info);
+
+		err = bpf_obj_get_info_by_fd(fd, &link_info, &link_info_len);
+		if (err) {
+			err = -errno;
+			pr_debug("Can't get link info for %u: %s", id, strerror(errno));
+			break;
+		}
+
+		if (link_info.type == BPF_LINK_TYPE_XDP && link_info.xdp.ifindex == ifindex && link_info.prog_id == prog_id) {
+			pr_debug("Detach link for id %u for prog %u on interface %u", id, prog_id, ifindex);
+			err = bpf_link_detach(fd);
+			if (err) {
+				err = -errno;
+				pr_warn("Can't detach link %u: %s", id, strerror(errno));
+			}
+			break;
+		}
+		close(fd);
+	}
+	close(fd);
+	return err;
+}
+
 static int xdp_multiprog__attach(struct xdp_multiprog *old_mp,
 				 struct xdp_multiprog *mp,
 				 enum xdp_attach_mode mode)
@@ -3209,8 +3298,13 @@ static int xdp_multiprog__attach(struct xdp_multiprog *old_mp,
 
 
 	err = xdp_attach_fd(prog_fd, old_fd, ifindex, mode);
-	if (err < 0)
+	if (err < 0) {
+		if (errno == EBUSY && !mp) {
+			pr_debug("Detaching link on ifindex %d\n", ifindex);
+			return xdp_detach_link(ifindex, xdp_multiprog__main_id(old_mp));
+		}
 		goto err;
+	}
 
 	if (mp)
 		pr_debug("Loaded %zu programs on ifindex %d%s\n",
