@@ -36,6 +36,7 @@
 #include "logging.h"
 
 #include "xdp_sample.skel.h"
+#include "xdp_load_bytes.skel.h"
 
 #define __sample_print(fmt, cond, ...)                                         \
 	({                                                                     \
@@ -184,6 +185,18 @@ bool sample_probe_cpumap_compat(void)
 	return res;
 }
 
+bool sample_probe_xdp_load_bytes(void)
+{
+	struct xdp_load_bytes *skel;
+	bool res;
+
+	skel = xdp_load_bytes__open_and_load();
+	res = !!skel;
+	xdp_load_bytes__destroy(skel);
+
+	return res;
+}
+
 void sample_check_cpumap_compat(struct bpf_program *prog,
 				struct bpf_program *prog_compat)
 {
@@ -1179,6 +1192,10 @@ static void stats_print(const char *prefix, int mask, struct stats_record *r,
 		sample_err_exp = false;
 		printf("\n");
 	}
+
+	fflush(stdout);
+	fflush(stderr);
+	// Flushing both outputs to "bypass" buffering
 }
 
 static int get_num_rxqs(const char *ifname)
@@ -1211,9 +1228,9 @@ static int get_num_rxqs(const char *ifname)
 	}
 
 	ret = ch.rx_count + ch.combined_count;
+	pr_debug("Got %d queues for ifname %s\n", ret, ifname);
 out:
 	close(fd);
-	pr_debug("Got %d queues for ifname %s\n", ret, ifname);
 	return ret;
 }
 
